package sun.misc;

import java.util.Enumeration;
import java.util.Dictionary;

public class Cache extends Dictionary
{
    private CacheEntry[] table;
    private int count;
    private int threshold;
    private float loadFactor;
    
    private void init(int n, float loadFactor) {
        if (n <= 0 || (double)loadFactor <= 0.0) {
            throw new IllegalArgumentException();
        }
        this.loadFactor = loadFactor;
        this.table = new CacheEntry[n];
        this.threshold = (int)((float)n * loadFactor);
        return;
    }
    
    public Cache(int n, float n2) {
        this.init(n, n2);
    }
    
    public Cache(int n) {
        this.init(n, 0.75f);
    }
    
    public Cache() {
        try {
            this.init(101, 0.75f);
        }
        catch (IllegalArgumentException ex) {
            throw new Error("panic");
        }
    }
    
    @Override
    public int size() {
        return this.count;
    }
    
    @Override
    public boolean isEmpty() {
        return this.count == 0;
    }
    
    @Override
    public synchronized Enumeration keys() {
        return new CacheEnumerator(this.table, true);
    }
    
    @Override
    public synchronized Enumeration elements() {
        return new CacheEnumerator(this.table, false);
    }
    
    @Override
    public synchronized Object get(Object obj) {
        CacheEntry[] table = this.table;
        int hashCode = obj.hashCode();
        CacheEntry next = table[(hashCode & Integer.MAX_VALUE) % table.length];
        while (next != null) {
            if (next.hash == hashCode && next.key.equals(obj)) {
                return next.check();
            }
            next = next.next;
        }
        return null;
    }
    
    protected void rehash() {
        int length = this.table.length;
        CacheEntry[] table = this.table;
        int n = length * 2 + 1;
        CacheEntry[] table2 = new CacheEntry[n];
        this.threshold = (int)((float)n * this.loadFactor);
        this.table = table2;
        while (length-- > 0) {
            CacheEntry next = table[length];
            while (next != null) {
                next = next.next;
                if (next.check() != null) {
                    int n2 = (next.hash & Integer.MAX_VALUE) % n;
                    next.next = table2[n2];
                    table2[n2] = next;
                }
                else {
                    --this.count;
                }
            }
        }
    }
    
    @Override
    public synchronized Object put(Object o, Object o2) {
        if (o2 == null) {
            throw new NullPointerException();
        }
        CacheEntry[] table = this.table;
        int hashCode = o.hashCode();
        int n = (hashCode & Integer.MAX_VALUE) % table.length;
        Object o3 = null;
        CacheEntry next = table[n];
        while (next != null) {
            if (next.hash == hashCode && next.key.equals(o)) {
                Object check = next.check();
                next.setThing(o2);
                return check;
            }
            if (next.check() == null) {
                o3 = next;
            }
            next = next.next;
        }
        if (this.count >= this.threshold) {
            this.rehash();
            return this.put(o, o2);
        }
        if (o3 == null) {
            o3 = new CacheEntry();
            o3.next = table[n];
            table[n] = o3;
            ++this.count;
        }
        o3.hash = hashCode;
        o3.key = o;
        o3.setThing(o2);
        return null;
    }
    
    @Override
    public synchronized Object remove(Object obj) {
        CacheEntry[] table = this.table;
        int hashCode = obj.hashCode();
        int n = (hashCode & Integer.MAX_VALUE) % table.length;
        CacheEntry next = table[n];
        CacheEntry cacheEntry = null;
        while (next != null) {
            if (next.hash == hashCode && next.key.equals(obj)) {
                if (cacheEntry != null) {
                    cacheEntry.next = next.next;
                }
                else {
                    table[n] = next.next;
                }
                --this.count;
                return next.check();
            }
            else {
                cacheEntry = next;
                next = next.next;
            }
        }
        return null;
    }
}
